/// The different types of Tokens.
#[derive(Clone, Copy, PartialEq, PartialOrd, Eq, Ord)]
pub enum TokenTypes
{
   /// A Comment token.
   Comment,

   /// An Identifier that has been confirmed
   /// to be a Keyword.
   Keyword,

   /// An alpha character followed by
   /// zero or more of alpha characters, numeric characters,
   /// or underscores. After the first character these can be
   /// mixed in any order.
   Identifier,

   /// A numeric character followed by
   /// zero or more numeric characters, zero or one period,
   /// and zero or one appendable type identifies.
   Numeric,

   /// A set of characters within double quote marks.
   /// These follow Rust's standard of escape characters.
   StaticString,

   /// This token is just whitespace,
   /// one or more space characters.
   Whitespace,

   /// This token has something, but we don't know what.
   /// This should not occur.
   Unknown
}



impl TokenTypes
{
   /// Get a str representation of this variant.
   pub fn to_str(&self) -> &'static str
   {
      match *self
      {
         TokenTypes::Comment =>
         {
            "Comment"
         }

         TokenTypes::Keyword =>
         {
            "Keyword"
         }

         TokenTypes::Identifier =>
         {
            "Identifier"
         }

         TokenTypes::Numeric =>
         {
            "Numeric"
         }

         TokenTypes::StaticString =>
         {
            "Unknown"
         }

         TokenTypes::Whitespace =>
         {
            "Whitespace"
         }

         TokenTypes::Unknown =>
         {
            "Unknown"
         }
      }
   }

   /// Get a String representation of this variant.
   pub fn to_string(&self) -> String
   {
      String::from(self.to_str())
   }
}

impl ::std::fmt::Debug for TokenTypes
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      write!(f, "{}", self.to_str())
   }
}

impl ::std::fmt::Display for TokenTypes
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      write!(f, "{}", self.to_str())
   }
}

