use std::path::PathBuf;

use ::compiler::read_file;
use ::reader::Reader;
use ::util::Util;



///
pub enum Parser
{
}



///
fn process_line(util: &Util, output: &mut Vec<String>, line: String)
{
   let start: usize;
   let end: usize;
   let mut include_file: PathBuf;
   let mut processed_line: String;

   // Copy the given line.
   processed_line = line.clone();

   // Remove any code block brackets.
   processed_line = processed_line.replace("{@", "");
   processed_line = processed_line.replace("@}", "");

   // Just handling the include code right now.
   if processed_line.contains("include") == true
   {
      // Parse one or more include statements on a line.

      // Cut out the include statement so it can be processed.
      match processed_line.find("\"")
      {
         Some(location) =>
         {
            // Add one to move past the quote mark.
            start = location + 1usize;
         }

         None =>
         {
            start = 0usize;
         }
      }

      match processed_line[start..].find("\"")
      {
         Some(location) =>
         {
            end = start + location;
         }

         None =>
         {
            end = start + 0usize;
         }
      }

      include_file = PathBuf::from(&processed_line[start..end]);

      // Try to find the include file in one of the
      // include directories.
      match util.get_include_file_path(&include_file)
      {
         Some(file) =>
         {
            // Process the include file found and add it to
            // the output list.
            output.push(read_file(util, &file));
         }

         None =>
         {
            error!("Unable to find desired include file:  {:?}",
                   include_file);
         }
      }
   }
   else
   {
      output.push(processed_line);
   }
}



impl Parser
{
   ///
   pub fn parse(util: &Util, reader: &mut Reader) -> Vec<String>
   {
      let mut output: Vec<String>;

      // Create a new set of lines to hold the parsed data.
      output = Vec::new();

      // Start parsing all the given lines.
      while reader.is_eob() == false
      {
         match reader.consume_line()
         {
            Ok(line) =>
            {
               process_line(util, &mut output, line)
            }

            Err(error) =>
            {
               error!("{}", error);
            }
         }
      }

      // Return the parsed input.
      output
   }
}
